//
//  NoodleSyntaxHighlighter.m
//  NoodleEdit
//
//  Created by Paul Kim on 5/6/12.
//  Copyright (c) 2012 Noodlesoft, LLC. All rights reserved.
//

#import "NoodleSyntaxHighlighter.h"

#define NOODLE_ELEMENT_TYPE_KEY @"noodleElementType"
#define NOODLE_COMMENT_TYPE     @"comment"
#define NOODLE_STRING_TYPE      @"string"

@implementation NoodleSyntaxHighlighter

@synthesize stringColor;
@synthesize commentColor;

- (id)initWithTextStorage:(NSTextStorage *)storage
{
    if ((self = [super init]) != nil)
    {
        textStorage = storage;
        [textStorage setDelegate:self];
        
        [self setStringColor:[NSColor redColor]];
        [self setCommentColor:[NSColor greenColor]];
    }
    return self;
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [self setStringColor:nil];
    [self setCommentColor:nil];
    [super dealloc];
}

- (void)parse:(id)sender
{
    NSString            *string;
    NSScanner           *scanner;
    NSCharacterSet      *startCharacters;
    NSRange             range;
    NSLayoutManager     *layoutManager;
    NSArray             *layoutManagerList;
    
    string = [textStorage string];
    
    
    layoutManagerList = [textStorage layoutManagers];
    range = NSMakeRange(0, [string length]);
    for (layoutManager in layoutManagerList)
    {
        [layoutManager setDelegate:self];
        [layoutManager removeTemporaryAttribute:NOODLE_ELEMENT_TYPE_KEY forCharacterRange:range];
    }
    
    scanner = [NSScanner scannerWithString:string];
    [scanner setCharactersToBeSkipped:nil];
    
    startCharacters  = [NSCharacterSet characterSetWithCharactersInString:@"\"/"];
    
    while (![scanner isAtEnd])
    {
        [scanner scanUpToCharactersFromSet:startCharacters intoString:NULL];
        
        if (![scanner isAtEnd])
        {
            range.location = [scanner scanLocation];
            range.length = 0;
            if ([scanner scanString:@"/" intoString:NULL])
            {
                if ([scanner scanString:@"*" intoString:NULL])
                {
                    if ([scanner scanUpToString:@"*/" intoString:NULL])
                    {
                        [scanner scanString:@"*/" intoString:NULL];
                    }
                    
                    range.length = [scanner scanLocation] - range.location;

                    for (layoutManager in layoutManagerList)
                    {
                        [layoutManager addTemporaryAttributes:[NSDictionary dictionaryWithObject:NOODLE_COMMENT_TYPE forKey:NOODLE_ELEMENT_TYPE_KEY] forCharacterRange:range];
                    }
                }
            }
            else if ([scanner scanString:@"\"" intoString:NULL])
            {
                if ([scanner scanUpToString:@"\"" intoString:NULL])
                {
                    [scanner scanString:@"\"" intoString:NULL];
                }
                range.length = [scanner scanLocation] - range.location;
                
                for (layoutManager in layoutManagerList)
                {
                    [layoutManager addTemporaryAttributes:[NSDictionary dictionaryWithObject:NOODLE_STRING_TYPE forKey:NOODLE_ELEMENT_TYPE_KEY] forCharacterRange:range];
                }
            }
        }
    }
}

#pragma mark NSTextStorage delegate methods

- (void)textStorageDidProcessEditing:(NSNotification *)aNotification
{
    [self performSelector:@selector(parse:) withObject:self afterDelay:0.0];
}


#pragma mark NSLayoutManager delegate methods

- (NSDictionary *)layoutManager:(NSLayoutManager *)layoutManager shouldUseTemporaryAttributes:(NSDictionary *)attrs forDrawingToScreen:(BOOL)toScreen atCharacterIndex:(NSUInteger)charIndex effectiveRange:(NSRangePointer)effectiveCharRange
{
    if (toScreen)
    {
        NSString                *type;
        NSColor                 *color;
        NSMutableDictionary     *returnDict;
        
        type = [attrs objectForKey:NOODLE_ELEMENT_TYPE_KEY];
        
        color = nil;
        if ([type isEqual:NOODLE_STRING_TYPE])
        {
            color = [self stringColor];
        }
        else if ([type isEqual:NOODLE_COMMENT_TYPE])
        {
            color = [self commentColor];
        }
        
        if (color != nil)
        {
            returnDict = [NSMutableDictionary dictionaryWithDictionary:attrs];
            [returnDict setObject:color forKey:NSForegroundColorAttributeName];
            return returnDict;
        }
    }
    return attrs;
}

@end
