//
//  NoodleTextDocument.m
//  NoodleEdit
//
//  Created by Paul Kim on 4/29/12.
//  Copyright 2012 Noodlesoft, LLC. All rights reserved.
//

#import "NoodleTextDocument.h"
#import "NoodleTextDocumentWindowController.h"
#import "NoodleSyntaxHighlighter.h"

@interface NoodleTextDocument (Private)

- (NSWindowController *)createWindowController;

@end

@implementation NoodleTextDocument

@synthesize contents;

- (id)init
{
    self = [super init];
    if (self) {
        // Add your subclass-specific initialization here.
        // If an error occurs here, send a [self release] message and return nil.
        contents = [[NSTextStorage alloc] init];
        syntaxHighlighter = [[NoodleSyntaxHighlighter alloc] initWithTextStorage:contents];
    }
    return self;
}

- (void)dealloc
{
    [contents release];
    [syntaxHighlighter release];
    [super dealloc];
}

- (void)makeWindowControllers
{
    [self newWindow:self];
}

- (void)windowControllerDidLoadNib:(NSWindowController *)aController
{
    [super windowControllerDidLoadNib:aController];
    // Add any code here that needs to be executed once the windowController has loaded the document's window.
}

- (NSData *)dataOfType:(NSString *)typeName error:(NSError **)outError
{
    /*
     Insert code here to write your document to data of the specified type. If outError != NULL, ensure that you create and set an appropriate error when returning nil.
    You can also choose to override -fileWrapperOfType:error:, -writeToURL:ofType:error:, or -writeToURL:ofType:forSaveOperation:originalContentsURL:error: instead.
    */
    return [[contents string] dataUsingEncoding:NSUTF8StringEncoding];
}

- (BOOL)readFromData:(NSData *)data ofType:(NSString *)typeName error:(NSError **)outError
{
    /*
    Insert code here to read your document from the given data of the specified type. If outError != NULL, ensure that you create and set an appropriate error when returning NO.
    You can also choose to override -readFromFileWrapper:ofType:error: or -readFromURL:ofType:error: instead.
    If you override either of these, you should also override -isEntireFileLoaded to return NO if the contents are lazily loaded.
    */
    NSString        *temp;
    
    temp = [[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] autorelease];
    contents = [[NSTextStorage alloc] initWithString:temp];
    [contents setFont:[NSFont userFixedPitchFontOfSize:[NSFont systemFontSize]]];
    [syntaxHighlighter release];
    syntaxHighlighter = [[NoodleSyntaxHighlighter alloc] initWithTextStorage:contents];

    return YES;
}

- (IBAction)newWindow:(id)sender
{
    [[self createWindowController] showWindow:self];
}

- (NSWindowController *)createWindowController
{
    NoodleTextDocumentWindowController      *controller;
    
    controller = [[NoodleTextDocumentWindowController alloc] initWithWindowNibName:@"NoodleTextDocument"];
    
    [self addWindowController:controller];
    return [controller autorelease];
}



+ (BOOL)autosavesInPlace
{
    return YES;
}

@end
